/*
 *
 *	This file is part of GLogiK project.
 *	GLogiK, daemon to handle special features on gaming keyboards
 *	Copyright (C) 2016-2018  Fabrice Delliaux <netbox253@gmail.com>
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation, either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __GLOGIK_GKDBUS_H__
#define __GLOGIK_GKDBUS_H__

#include <string>
#include <vector>
#include <sstream>

#include <dbus/dbus.h>

#include "lib/utils/utils.h"

#include "GKDBusConnection.h"
#include "GKDBusEvents.h"

#include "messages/GKDBusRemoteMethodCall.h"
#include "messages/GKDBusBroadcastSignal.h"
#include "messages/GKDBusTargetsSignal.h"

#include "arguments/GKDBusArgString.h"
#include "arguments/GKDBusArgBoolean.h"
#include "arguments/GKDBusArgByte.h"
#include "arguments/GKDBusArgUInt16.h"
#include "arguments/GKDBusArgMacro.h"

namespace NSGKDBus
{

using namespace NSGKUtils;

class GKDBusEventFound : public GLogiKExcept
{
	public:
		GKDBusEventFound(const std::string & msg = "event found") : GLogiKExcept(msg) {};
		virtual ~GKDBusEventFound( void ) throw() {};
};

class GKDBus
	:	public GKDBusEvents,
		virtual public GKDBusMessageRemoteMethodCall,
		public GKDBusMessageBroadcastSignal,
		public GKDBusMessageTargetsSignal,
		virtual public GKDBusArgumentString,
		public GKDBusArgumentBoolean,
		virtual public GKDBusArgumentByte,
		virtual public GKDBusArgumentUInt16,
		public GKDBusArgumentMacro
{
	public:
		GKDBus(const std::string & rootnode);
		~GKDBus();

		void connectToSystemBus(const char* connection_name);

		void checkForNextMessage(const BusConnection bus);

		void appendExtraStringToMessage(const std::string & value);

	protected:

	private:
		std::ostringstream buffer_;
		DBusError error_;

		DBusConnection* session_conn_;
		DBusConnection* system_conn_;

		std::string session_name_;
		std::string system_name_;

		DBusMessage* message_;

		void checkDBusError(const char* error_message);
		void checkReleasedName(int ret);
		DBusConnection* getConnection(BusConnection bus_connection);
};

} // namespace NSGKDBus

#endif
